#include <stdio.h>
#include <string.h>

#define ENTRY(X)	list[X].name[0]
#define NAME(X) 	list[X].name[0]
#define NAME1(X)	list[X].name[1]
#define PHONE(X)	list[X].phone[0]
#define PHONE1(X)	list[X].phone[1]

#define NAME_LN 	list[X].name[0]
#define PHONE_LN	list[X].phone[0]
#define BAUD(X) 	CTOI(list[X].baud[0]) + CTOI(list[X].baud[1]) * 256
#define DATA(X) 	list[X].data[0]
#define STOP(X) 	list[X].stop[0]
#define PARITY(X)	parity[list[X].prty]

#define ENTRY_L 	48
#define NAME_L		26
#define NAME1_L 	25
#define PHONE_L 	15
#define PHONE1_L	14
#define BAUD_L		2
#define DATA_L		2
#define STOP_L		2
#define PARITY_L	1

#define DIRCNT		200
#define DIRSIZE 	9600

#define CTOI(X) 	((X + 256) % 256)
#define COMP(X, Y, T)	comp(&ENTRY(X), &ENTRY(Y), T)
#define EMPTY(X)	empty(&NAME1(X))
#define SWAP(X, Y)	swap(&ENTRY(X), &ENTRY(Y))
#define FP		fprintf
#define PF		printf

struct	entry
{
	char	name[NAME_L];			/* Name of Bulletin Board */
	char	phone[PHONE_L]; 		/* Phone Number of Board */
	char	baud[BAUD_L];			/* Baud Rate */
	char	data[DATA_L];			/* Number of Data Bits */
	char	stop[STOP_L];			/* Number of Stop Bits */
	char	prty;				/* Parity 0=N, 1=E 2=O */
};




main (argc, argv)

int	argc;
char	*argv[];

{
	struct entry	list[DIRCNT];		/* QMODEM phone directory */

	static	char	name_fon[13]	= "qmodem.fon";
	static	char	name_bak[13]	= "qmodem.bak";
	static	char	parity[3]	= { 'N', 'E', 'O' };

	char	temp[256];
	char	filename[256];
	char	bkupname[256];

	char	*file, *bkup;
	char	*copt, *opt;
	char	*pd, *pt;
	int	fp;
	int	i, j, k, l, m, n;




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver processes the command line.	*/
/*	If the command was entered with no argumnets, or with too	*/
/*	many arguements, then procedure explain will be invoked to	*/
/*	describe the proper command syntax to the user.  This		*/
/*	program expects to receive either one or two arguments. 	*/
/*									*/
/*	This first argument is required, and it is used to determine	*/
/*	the type of processing that is to be performed on the QMODEM	*/
/*	phone directory.						*/
/*									*/
/*	The second argument is optional, and it is used to qualify	*/
/*	the drive, subdirectory, and path delimiter which are		*/
/*	required to locate the QMODEM.FON file, if the file is		*/
/*	not available in the default drive and directory.		*/
/*									*/
/*----------------------------------------------------------------------*/

	switch (argc)
	{

		case 2:
			copt = argv[1];
			file = &name_fon[0];
			bkup = &name_bak[0];
			break;

		case 3:
			copt = argv[1];
			file = makefnam(name_fon, argv[2], filename);
			bkup = makefnam(name_bak, argv[2], bkupname);
			break;

		default:
			explain();
			exit(0);
			break;
	}




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver reads the content of the		*/
/*	QMODEM directory.  The file will be opened for BUPDATE		*/
/*	if the 'q' option was not specified.  This will allow           */
/*	the process to abort the task if the file is marked as		*/
/*	READ ONLY.  If however, the 'q' option was specified, then      */
/*	the file will be opened for BREAD, since it will not be 	*/
/*	modified.							*/
/*									*/
/*----------------------------------------------------------------------*/

	FP(stderr, "\nProcessing file %s.\n\tLoading ...", file);

	m = strchr(copt, 'q') ? BUPDATE : BREAD;

	if ((fp = open(file, m)) < 0)
	{
		FP(stderr, "\nCan not open file %s.\n", file);
		exit(1);
	}


	for (l = DIRSIZE, pd = &ENTRY(0); l > 0; l -= i, pd += i)
		if ((i = read(fp, pd, l)) <= 0) break;


	close(fp);


	if (i < 0)
	{
		FP(stderr, "\nError in reading from file %s.\n", file);
		exit(1);
	}


	if (l > 0)
	{
		FP(stderr, "\nPremature EOF in file %s.\n", file);
		exit(1);
	}




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver eliminates all embeded empty	*/
/*	entries from the QMODEM directory.  Whenever an embeded 	*/
/*	empty entry is found, it will be replaced with a non-empty	*/
/*	entry from the bottom of the directory list.			*/
/*									*/
/*----------------------------------------------------------------------*/


	FP(stderr, "\b\b\b\b, Processing ...");


	/* Scan from the first entry toward the last entry */

	for (i = 0, n = DIRCNT - 1; i <= n; i++)
	{
		if (EMPTY(i) == 0) continue;


		/* Scan from the last entry toward the first entry */

		for (j = n; j > i; j--)
		{
			n--;

			if (EMPTY(j)) continue;

			SWAP(i, j);
			break;

		}


		if (j <= i) n--;

	}


	/* The QMODEM phone directory is empty */

	if (n < 0)
	{
		FP(stderr, "\nQMODEM phone directory is empty.\n");
		exit(1);
	}




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver performs a bubble sort on		*/
/*	the QMODEM directory entries.  This method of sorting is	*/
/*	employed because the sample size is small.			*/
/*									*/
/*----------------------------------------------------------------------*/


	/* Scan from the first entry toward the last entry */

	for (i = 0; i < n; i++)
	{


		/* Scan from the last entry toward the first entry */

		for (j = n, k = i; j > i; j--)
		{


			/*----------------------------------------------*/
			/* Compare the two entries according to the	*/
			/* fields selected by the user on the first	*/
			/* command argument				*/
			/*----------------------------------------------*/

			for (opt = copt; *opt != NULL; opt++)
			{
				m = COMP(k, j, *opt);
				if (m != 0) break;
			}


			/*----------------------------------------------*/
			/* This entry is a candiddate to be exchanged	*/
			/* with the top entry				*/
			/*----------------------------------------------*/

			if (m > 0) k = j;


		}


		/* Interchange the last candidate entry with the top entry */

		if (k > i) SWAP(k, j);

	}




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver will not do anything if the	*/
/*	'q' option was specified.  Otherwise, it will perform the       */
/*	following operations: rename the original QMODEM.FON to 	*/
/*	QMODEM.BAK and retain this file as a backup; create a new	*/
/*	QMODEM.FON; write the contents of the sorted directory		*/
/*	into QMODEM.FON; delete the backup file QMODEM.BAK, unless	*/
/*	the 'r' option was specified, then the backup file is kept.     */
/*									*/
/*----------------------------------------------------------------------*/

	if (strchr(copt, 'q') == NULL)
	{
		FP(stderr, "\b\b\b\b, Saving ...");


		if (rename(file, bkup) != 0)
		{
			FP(stderr, "\nUnable to create %s.\n", bkup);
			exit(1);
		}

		if ((fp = creat(file, BWRITE)) < 0)
		{
			FP(stderr, "\nUnable to recreate file %s.\n", file);
			exit(1);
		}


		for (l = DIRSIZE, pd = &ENTRY(0); l > 0; l -= i, pd += i)
			if ((i = write(fp, pd, l)) <= 0) break;

		close(fp);


		if (i < 0)
		{
			FP(stderr, "\nError in writing file %s.\n", file);
			exit(1);
		}

		if (l > 0)
		{
			FP(stderr, "\nIncomplete data in file %s.\n", file);
			exit(1);
		}


		if (strchr(copt, 'r') == NULL) unlink(bkup);

	}


	FP(stderr, "\b\b\b\b, Done.\n");




/*----------------------------------------------------------------------*/
/*									*/
/*	This part of the main driver will not do anything unless	*/
/*	either the 'l' or 'q' option was specified.  If either          */
/*	option was specified, then this tak will display the		*/
/*	sorted QMODEM directory and pause after every 20th entries.	*/
/*	The pause is performed to give the user an opportunity to	*/
/*	review the data on the display, and to issue a response 	*/
/*	which will either allow this task to continue, or terminate	*/
/*	it.								*/
/*									*/
/*----------------------------------------------------------------------*/


	if ((strchr(copt, 'l') != NULL) | (strchr(copt, 'q') != NULL))
	{
		for (i = 0, j = 1; i <= n; i++)
		{
			if ((i % 20) == 0)
			{
				if (i > 0) more();

				crt_cls();

				PF("\t\tQMODEM directory\t\t\tPage %2d\n\n",
				   j++);
			}

			m = NAME1_L + PHONE1_L +2;

			for (l = 0; l < m; l++) temp[l] = ' ';

			temp[m] = NULL;
			l = PHONE(i);

			strncpy(&temp[0], &NAME1(i), NAME1_L);
			strncpy(&temp[m - l], &PHONE1(i), l);

			PF("\t%3d)  %40s  %4d-%1d-%c-%1d\n",
			       i + 1, temp,
			       BAUD(i), DATA(i), PARITY(i), STOP(i));

		}

	}


	exit(0);


}




explain()

/*----------------------------------------------------------------------*/
/*									*/
/*	This function is used to provide the user with a description	*/
/*	of this utility, and to identify its author.			*/
/*									*/
/*----------------------------------------------------------------------*/

{

 crt_cls();
 PF("QS - QMODEM Phone Directory Sorter, Version 1.01 - 10/01/85\n");
 PF("Copyright(c) 1985, GEE WIZ(tm) Software Company, ALL RIGHTS RESERVED\n\n");

 PF("Written by Gee M. Wong for Public Domain use ONLY.\n\n");

 PF("QS may not be sold, used in a commerical environment, nor packaged\n");
 PF("with any commerical product without prior written permission from the\n");
 PF("GEE WIZ Software Company.\n\n");

 PF("QS may be distributed freely at no cost to the recipient.  User\n");
 PF("support for this utility may be best obtained by notifing GEE WIZ.\n");
 PF("NO SUBSCRIPTION IS NECESSARY FOR USER SUPPORT.  GEE WIZ encourages all\n");
 PF("user feedback.\n\n");

 PF("QS is a utility which permits a user to sort the QMODEM(tm) telephone\n");
 PF("directory using one or more key fields in any order.  This version of\n");
 PF("QS supports sorting on: area code, area code prefix, baud rate, name,\n");
 PF("and telephone number.\n\n");

 PF("QS is written in C86(tm) and is designed to execute on an IBM PC or a\n");
 PF("true compatable with MS-DOS(tm) or PC-DOS(tm) versions 2.0 or higher.\n");

 more();

 crt_cls();
 PF("The command format for QS is:\n\n");

 PF("\tQS  [a][b][l][n][p][q][r][t]  [[drive:][path]QMODEM[.FON]]\n\n");

 PF("One or more of the following options may be specified in any order:\n\n");

 PF("\ta\tDefine the area code as a sort field\n");
 PF("\tb\tDefine the baud rate as a sort field\n");
 PF("\tl\tList the processed directory on the console\n");
 PF("\tn\tDefine the name field as a sort field\n");
 PF("\tp\tDefine the one digit area code prefix as a sort field\n");
 PF("\tq\tQuery the directory, no updates will be made to QMODEM.FON\n");
 PF("\tr\tRetain the original directory in QMODEM.BAK\n");
 PF("\tt\tDefine the entire telephone number as a sort field\n\n");

 PF("The file specification is optional.  You would only need to use this\n");
 PF("parameter if your QMODEM.FON file is located somewhere other than the\n");
 PF("default drive or directory.  Furthermore, you do not have to specify\n");
 PF("the file extension FON, QS will always assume the extension is FON.\n");

 more();

 crt_cls();
 PF("Whenever QS performs any operation which requires updating the QMODEM\n");
 PF("phone directory on disk, the original QMODEM.FON file will be renamed\n");
 PF("to QMODEM.BAK before a new QMODEM.FON is written.  If all goes well,\n");
 PF("then QMODEM.BAK will be deleted.  However, if either an error occurs\n");
 PF("or if the 'r' options is used, then QMODEM.BAK will be retained.\n\n");

 PF("If you feel that this utility is of value, please feel free to send\n");
 PF("your generous contribution of $1.00 CASH to:\n\n");

 PF("\t\tGEE WIZ Software Company\n");
 PF("\t\t10 Manton Avenue\n");
 PF("\t\tEast Brunswick, NJ 08816\n");
 PF("\t\tc/o: Janet Huie\n\n");

 PF("NOTE:\tGEE WIZ is a trademark of GEE WIZ Software Company.\n");
 PF("\tQMODEM is a trademark of the Forbin Project.\n");
 PF("\tC86 is a trademark of Computer Innovations, Incorporated.\n");
 PF("\tMS-DOS is a trademark of Microsoft Incorporated.\n");
 PF("\tPC-DOS is a trademark of Microsoft Incorporated.\n");
 PF("\tPC-DOS is a trademark of International Business Machines.\n");


}




more()

/*----------------------------------------------------------------------*/
/*									*/
/*	This function is invoked during the QMODEM directory listing,	*/
/*	and the QS utility description process, to pause the		*/
/*	displaying of data.  This allows the user to view the data	*/
/*	before it can roll off the screen, and it also allows the user	*/
/*	to decide whether to continue or terminate this utility.	*/
/*									*/
/*----------------------------------------------------------------------*/

{
	int	i;
	char	c;

	FP(stderr, "\n\n----- More (y) or n ? -----");

	i = key_getc();
	c = i & 0x00ff;

	if (tolower(c) == 'n') exit(0);
}




empty(x)
char	*x;

/*----------------------------------------------------------------------*/
/*									*/
/*	This function is used to determine if a QMODEM directory	*/
/*	is empty or not.  If the QMODEM directory entry is empty,	*/
/*	the the name field should contain nothing but '-' characters.   */
/*	This function expects X to be pointing to the name field on	*/
/*	entry.								*/
/*									*/
/*	The following values will be returned by this function: 	*/
/*									*/
/*	    0	=   The QMODEM directory entry is an empty entry	*/
/*	    1	=   The Qmodem directory entry contains data		*/
/*									*/
/*----------------------------------------------------------------------*/

{

	int	i;

	for (i = 0; i < NAME1_L; i++)
		if (*x++ != '-') return(0);

	return(1);

}




swap(x, y)
char	*x, *y;

/*----------------------------------------------------------------------*/
/*									*/
/*	This function is used to interchange the contents of two	*/
/*	QMODEM directory entries.					*/
/*									*/
/*	This function expects X and Y to be pointing to the beginning	*/
/*	of a QMODEM directory entry.					*/
/*									*/
/*----------------------------------------------------------------------*/

{

	int	i;
	char	c;

	for (i = 0; i < ENTRY_L; i++)
	{
		c    = *x;
		*x++ = *y;
		*y++ = c;
	}

}




comp(x, y, t)
struct	entry	*x, *y;
char	t;

/*----------------------------------------------------------------------*/
/*									*/
/*	This function is used to compare two QMODEM directory entries.	*/
/*									*/
/*	This function expects X and Y to be pointing to the beginning	*/
/*	of a QMODEM directory entry, and T to contain a character	*/
/*	which signifies which fields are to be compared.		*/
/*									*/
/*	This function returns the following values:			*/
/*									*/
/*	    -1	 =   The field in X is less than the field in Y 	*/
/*	     0	 =   The field in X is equal to the field in Y		*/
/*	     1	 =   The field in X is greater than the field in Y	*/
/*									*/
/*----------------------------------------------------------------------*/

{

	int	ix, iy;

	switch(tolower(t))
	{

		case 'a':
			ix = x->phone[0] - 11;
			iy = y->phone[0] - 11;

			if (ix <= 0) return((iy <= 0) ? 0 : -1);

			if (iy <= 0) return(1);

			return(strncmp(&x->phone[ix], &y->phone[iy], 3));

		case 'b':
			ix = CTOI(x->baud[0]) + CTOI(x->baud[1]) * 256;
			iy = CTOI(y->baud[0]) + CTOI(y->baud[1]) * 256;

			if (ix == iy) return(0);

			return((ix > iy) ? 1 : -1);

		case 'n':
			return(strncmp(&x->name[1], &y->name[1], NAME1_L));

		case 'p':
			ix = x->phone[0] - 13;
			iy = y->phone[0] - 13;

			if (ix <= 0) return((iy <= 0) ? 0 : -1);

			if (iy <= 0) return(1);

			if (x->phone[ix] == y->phone[iy]) return(0);

			return((x->phone[ix] > y->phone[iy]) ? 1 : -1);

		case 't':
			return(strncmp(&x->phone[0], &y->phone[0], PHONE_L));

		default:
			return(0);

	}

}
